# coding: utf-8
import autopath
from tiramisu import setting
setting.expires_time = 1
from tiramisu.option import IntOption, OptionDescription
from tiramisu.config import Config
from time import sleep


def make_description():
    u1 = IntOption('u1', '', multi=True)
    u2 = IntOption('u2', '')
    u3 = IntOption('u3', '', multi=True)
    return OptionDescription('od1', '', [u1, u2, u3])


def test_cache():
    od1 = make_description()
    c = Config(od1)
    values = c.cfgimpl_get_values()
    settings = c.cfgimpl_get_settings()
    c.u1
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    c.u2
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    assert 'u2' in values._p_.get_cached('value', c)
    assert 'u2' in settings._p_.get_cached('property', c)


def test_cache_reset():
    od1 = make_description()
    c = Config(od1)
    values = c.cfgimpl_get_values()
    settings = c.cfgimpl_get_settings()
    #when change a value
    c.u1
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    c.u2 = 1
    assert 'u1' not in values._p_.get_cached('value', c)
    assert 'u1' not in settings._p_.get_cached('property', c)
    #when remove a value
    c.u1
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    del(c.u2)
    assert 'u1' not in values._p_.get_cached('value', c)
    assert 'u1' not in settings._p_.get_cached('property', c)
    #when add/del property
    c.u1
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    c.cfgimpl_get_settings()[od1.u2].append('test')
    assert 'u1' not in values._p_.get_cached('value', c)
    assert 'u1' not in settings._p_.get_cached('property', c)
    c.u1
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    c.cfgimpl_get_settings()[od1.u2].remove('test')
    assert 'u1' not in values._p_.get_cached('value', c)
    assert 'u1' not in settings._p_.get_cached('property', c)
    #when enable/disabled property
    c.u1
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    c.cfgimpl_get_settings().append('test')
    assert 'u1' not in values._p_.get_cached('value', c)
    assert 'u1' not in settings._p_.get_cached('property', c)
    c.u1
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    c.cfgimpl_get_settings().remove('test')
    assert 'u1' not in values._p_.get_cached('value', c)
    assert 'u1' not in settings._p_.get_cached('property', c)


def test_cache_reset_multi():
    od1 = make_description()
    c = Config(od1)
    values = c.cfgimpl_get_values()
    settings = c.cfgimpl_get_settings()
    #when change a value
    c.u1
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    c.u3 = [1]
    assert 'u1' not in values._p_.get_cached('value', c)
    assert 'u1' not in settings._p_.get_cached('property', c)
    #when append value
    c.u1
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    c.u3.append(1)
    assert 'u1' not in values._p_.get_cached('value', c)
    assert 'u1' not in settings._p_.get_cached('property', c)
    #when pop value
    c.u1
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    c.u3.pop(1)
    assert 'u1' not in values._p_.get_cached('value', c)
    assert 'u1' not in settings._p_.get_cached('property', c)
    #when remove a value
    c.u1
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    del(c.u3)
    assert 'u1' not in values._p_.get_cached('value', c)
    assert 'u1' not in settings._p_.get_cached('property', c)


def test_reset_cache():
    od1 = make_description()
    c = Config(od1)
    values = c.cfgimpl_get_values()
    settings = c.cfgimpl_get_settings()
    c.u1
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    c.cfgimpl_reset_cache()
    assert 'u1' not in values._p_.get_cached('value', c)
    assert 'u1' not in settings._p_.get_cached('property', c)
    c.u1
    sleep(1)
    c.u2
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    assert 'u2' in values._p_.get_cached('value', c)
    assert 'u2' in settings._p_.get_cached('property', c)
    c.cfgimpl_reset_cache()
    assert 'u1' not in values._p_.get_cached('value', c)
    assert 'u1' not in settings._p_.get_cached('property', c)
    assert 'u2' not in values._p_.get_cached('value', c)
    assert 'u2' not in settings._p_.get_cached('property', c)


def test_reset_cache_only_expired():
    od1 = make_description()
    c = Config(od1)
    values = c.cfgimpl_get_values()
    settings = c.cfgimpl_get_settings()
    c.u1
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    c.cfgimpl_reset_cache(True)
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    sleep(1)
    c.u2
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    assert 'u2' in values._p_.get_cached('value', c)
    assert 'u2' in settings._p_.get_cached('property', c)
    c.cfgimpl_reset_cache(True)
    assert 'u1' not in values._p_.get_cached('value', c)
    assert 'u1' not in settings._p_.get_cached('property', c)
    assert 'u2' in values._p_.get_cached('value', c)
    assert 'u2' in settings._p_.get_cached('property', c)


def test_reset_cache_only():
    od1 = make_description()
    c = Config(od1)
    values = c.cfgimpl_get_values()
    settings = c.cfgimpl_get_settings()
    c.u1
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    c.cfgimpl_reset_cache(only=('values',))
    assert 'u1' not in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    c.u1
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' in settings._p_.get_cached('property', c)
    c.cfgimpl_reset_cache(only=('settings',))
    assert 'u1' in values._p_.get_cached('value', c)
    assert 'u1' not in settings._p_.get_cached('property', c)
